package fi.flexplex.lib;

import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Optional;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Logger;

import com.neovisionaries.ws.client.WebSocketException;

public final class FlexLib {

	private final FlexLib instance;
	private final String token;
	private final Logger logger;
	private final Timer timer;
	private final TimerTask timerTask;
	private final GraphQLApi graphQLApi;
	private final Set<FlexLibAdapter> eventListeners = new HashSet<>();
	private WebSocketClient webSocketClient;
	private final HashMap<String, PlayerFriends> playerFriends = new HashMap<>();

	protected Set<FlexLibAdapter> getEventListeners() {
		return this.eventListeners;
	}

	protected void updatePlayerFriends(final String player) {
		final PlayerFriends pFriends = this.playerFriends.get(player);
		if (pFriends != null) {
			pFriends.update();
		}
	}

	/**
	 * Instantiates FlexLib API
	 * @param token token for FlexPlex API
	 */
	public FlexLib(final String token, final Logger logger) {
		logger.info("Initializing FlexLib");
		this.instance = this;
		this.token = token;
		this.logger = logger;
		this.graphQLApi = new GraphQLApi(this, "https://api.flexplex.fi/graphql");

		new FlexLibAdapterImpl(this);

		this.timer = new Timer();
		this.timerTask = new TimerTask() {
			@Override
			public void run() {
				final long removeTime = System.currentTimeMillis() - 10_800_000;
				playerFriends.entrySet().removeIf(entry -> entry.getValue().getLastUsed() < removeTime);

				// Connect websocket if closed
				if (webSocketClient != null && !webSocketClient.isOpen()) {
					webSocketClient.disable();
					webSocketClient = null;
				}
				if (webSocketClient == null) {
					try {
						webSocketClient = new WebSocketClient(instance, "wss://api.flexplex.fi/flexlib", token);
						webSocketClient.connect();
					} catch (final IOException | WebSocketException e) {
						logger.warning("Failed to initialize FlexLib WebSocket client");
					}
				}
			}
		};
		this.timer.schedule(this.timerTask, 0, 60_000);
	}

	/**
	 * Get API token
	 * @return API Token used to connect FlexPlex
	 */
	public String getToken() {
		return this.token;
	}

	/**
	 * Get logger used by FlexLib
	 * @return FlexLib logger
	 */
	public Logger getLogger() {
		return this.logger;
	}

	/**
	 * Add new event listener
	 * @param listener
	 */
	public void addListener(final FlexLibAdapter listener) {
		this.eventListeners.add(listener);
	}

	/**
	 * Get Friends API
	 * @return Friends API if available
	 */
	public Optional<PlayerFriends> getPlayerFriends(final String player) {
		PlayerFriends pFriends = this.playerFriends.get(player);
		if (pFriends == null) {
			try {
				pFriends = new PlayerFriends(this.graphQLApi, this.token, player);
			} catch (final IOException e) {
				return Optional.empty();
			}
			this.playerFriends.put(player, pFriends);
		}
		return Optional.of(pFriends);
	}

	/**
	 * Disableds FlexLib API instance
	 */
	public void disable() {
		this.timerTask.cancel();
		this.timer.cancel();

		if (this.webSocketClient != null) {
			this.webSocketClient.disable();
		}

		this.eventListeners.clear();
		this.playerFriends.clear();
	}

}
